/*
 * sbmixer.c - Copyright (C) 1995 Pat Thoyts <pat@calubc.caladan.co.uk>
 *
 * $Id: sbmixer.c,v 0.2 1997/02/05 23:05:34 pat Exp $
 */

#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "sbmixer.h"

const unsigned char *gnutext=
"
   Copyright (C) 1995 Pat Thoyts <pat@calubc.caladan.co.uk>

mixer is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

This program  is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; see the file COPYING.  If not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

";
const unsigned char *internal_name = "sbmixer";
const unsigned char *version_string = "0.2.0";

struct flag_struct flag = {0, 0, 0, {{0,0},{0,0},{0,0}}, 0};

int write_levels(int mixer, struct mixer_struct *device, int nr_devices);
int interactive (int mixer, struct mixer_struct *device, int nr_devices);
int show_levels(int mixer, struct mixer_struct *device, int nr_devices); 
int init(int mixer, struct mixer_struct *device);


int
main(int argc, char **argv)
{
  int mixer, nr_devices;
  struct mixer_struct device[SOUND_MIXER_NRDEVICES];

  get_options(argc, argv, device);
  if(flag.debug) {
    fprintf(stderr, "DEBUG: Debugging mode on.\n");
    if(flag.large_range)
      fprintf(stderr, "DEBUG: Using 0-255 range for levels.\n");
    if(flag.verbose) fprintf(stderr, "DEBUG: Verbose mode.\n");
  }
  if ((mixer = open(MIXER_DEVICE, O_RDWR, 0 )) == -1) {
    fprintf(stderr, "Oops. Failed to open the mixer.\n");
    exit(1);
  }
  nr_devices = init(mixer, device);
  if(flag.nr_changes != 0)
    write_levels(mixer, device, nr_devices);
  if(argc < 2) {
    show_levels(mixer, device, nr_devices);
    /* interactive(mixer, device, nr_devices); */
  }
  else {
    if (flag.verbose)
      show_levels(mixer, device, nr_devices);
  }
  close(mixer);
  return 0;
}

int
write_levels(int mixer, struct mixer_struct *device, int nr_devices)
{
  int n, cn, newleft, newright, new;
  const char *name;
  const char ids[] = {'m','b','t','i','v','k','l','C','c','x','2','r'};
  const char *names[] = SOUND_DEVICE_NAMES;
  
  for(n=0; n <= flag.nr_changes - 1; n++) {
    name = '\0';
    if(flag.debug) fprintf(stderr,
	  "WRITE: %c, change to %d\n", flag.change[n].id, flag.change[n].new);
    for(cn = 0; cn < SOUND_MIXER_NRDEVICES; cn++) {
      if (ids[cn] == flag.change[n].id) name = names[cn]; 
    }
    if(name == '\0') {
      fprintf(stderr, "Oops. Unrecognised device identifier `%c\'.\n",
	      flag.change[n].id);
    }
    else {
      for (cn = 0; cn < nr_devices; cn++) {
	if (device[cn].name == name) {
	  new = flag.change[n].new;
	  if (flag.large_range) {
	    newleft = new/256; newright = new%256;
	    newleft *= 2;      newright *= 2;
	    newleft /= 5;      newright /= 5;
	    new = (newleft*256)+newright;
	  }
	  if (ioctl(mixer, MIXER_WRITE(device[cn].id), &new) == -1) {
	    fprintf(stderr, "Oops. ioctl() call failed to write new value for"
		    " device `%s\'.\n", device[cn].name);
	  }
	}
      }
    }
  }
  return 1;
}

int
interactive (int mixer, struct mixer_struct *device, int nr_devices)
{
  int new;

  while(1) {
    show_levels(mixer, device, nr_devices);
    printf("Enter new volume (0-255): ");
    scanf("%d", &new);
    if (new < 256) {
      new = (new*256)+new;
      if (ioctl(mixer, SOUND_MIXER_WRITE_VOLUME, &new) == -1) {
	perror("Oops. Failed to write new value.\n");
	close(mixer);
	exit(2);
      }
    }
    else
      break;
  }
  return 1;
}

int
show_levels(int mixer, struct mixer_struct *device, int nr_devices)
{
  int n;
  for (n = 0; n < nr_devices; n++) {
    if (ioctl(mixer, MIXER_READ(device[n].id), &device[n].level) == -1) {
      fprintf(stderr, "Uhoh. Failed to read mixer device `%s\'.\n",
	      device[n].label);
      exit(2);
    }
    printf("%s: ", device[n].label);
    if (device[n].stereo)
      printf("%d %d\n", device[n].level/256, device[n].level%256);
    else
      printf(" %d\n", device[n].level/256);
  }
  return 1;
}

int
init(int mixer, struct mixer_struct *device)
{
  int n, mask = 0, stereomask = 0, dev_no = 0, recmask = 0, capsmask = 0;
  const char *label[] = SOUND_DEVICE_LABELS;
  const char *name[] = SOUND_DEVICE_NAMES;

  if (ioctl(mixer, SOUND_MIXER_READ_DEVMASK, &mask) == -1) {
    fprintf(stderr, "Uhoh. Failed to read mixer bitmask.\n");
    exit(2);
  }
  if (ioctl(mixer, SOUND_MIXER_READ_RECMASK, &recmask) == -1) {
    fprintf(stderr, "Uhoh. Failed to read recording bitmask.\n");
    exit(2);
  }
  if (ioctl(mixer, SOUND_MIXER_READ_STEREODEVS, &stereomask) == -1) {
    fprintf(stderr, "Uhoh. Failed to read stereo bitmask.\n");
    exit(2);
  }
  if (ioctl(mixer, SOUND_MIXER_READ_CAPS, &capsmask) == -1) {
    fprintf(stderr, "Uhoh. Failed to read caps bitmask.\n");
  }
  if(flag.debug) fprintf(stderr,
	         "DEBUG: device mask 0x%X, stereo device mask 0x%X\n"
		 "DEBUG: recordable device mask 0x%X, caps mask 0x%X\n",
		 mask, stereomask, recmask, capsmask);
  for (n = 0; n < SOUND_MIXER_NRDEVICES; n++) {
    if ( mask & (1 << n) ) {
      device[dev_no].id = n;
      device[dev_no].name = name[n];
      device[dev_no].label = label[n];
      if ( stereomask & (1 << n))
	device[dev_no].stereo = 1;
      else
	device[dev_no].stereo = 0;
      if ( recmask & (1 << n))
	device[dev_no].can_record = 1;
      else
	device[dev_no].can_record = 0;
      if (flag.debug)
	fprintf(stderr, "DEBUG: Supports %s (0x%X) as a %s%sdevice.\n",
		device[dev_no].name, device[dev_no].id,
		device[dev_no].stereo ? "stereo" : "mono",
		device[dev_no].can_record ? "recording " : " ");
      dev_no++;
    }
    else
      if (flag.debug)
	fprintf(stderr, "DEBUG: Doesn't support %s (0x%X).\n", name[n], n);
  }   
  return dev_no;
}
